package kr.co.vividnext.sodalive.audio_content.comment

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class AudioContentCommentListViewModel(
    private val repository: AudioContentCommentRepository
) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _commentList = MutableLiveData<List<GetAudioContentCommentListItem>>()
    val commentList: LiveData<List<GetAudioContentCommentListItem>>
        get() = _commentList

    private var _totalCommentCount = MutableLiveData(0)
    val totalCommentCount: LiveData<Int>
        get() = _totalCommentCount

    var page = 1
    private var isLast = false
    private val size = 10

    fun getCommentList(audioContentId: Long, onFailure: (() -> Unit)? = null) {
        if (!_isLoading.value!! && !isLast) {
            _isLoading.value = true
            compositeDisposable.add(
                repository.getAudioContentCommentList(
                    audioContentId = audioContentId,
                    page = page,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            if (it.success && it.data != null) {
                                _totalCommentCount.postValue(it.data.totalCount)

                                if (it.data.items.isNotEmpty()) {
                                    page += 1
                                    _commentList.postValue(it.data.items)
                                } else {
                                    isLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.postValue(it.message)
                                } else {
                                    _toastLiveData.postValue(
                                        "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                    )
                                }

                                if (onFailure != null) {
                                    onFailure()
                                }
                            }

                            _isLoading.value = false
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                            if (onFailure != null) {
                                onFailure()
                            }
                        }
                    )
            )
        }
    }

    fun registerComment(contentId: Long, comment: String) {
        if (!_isLoading.value!!) {
            _isLoading.value = true
        }

        compositeDisposable.add(
            repository.registerComment(
                contentId = contentId,
                comment = comment,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false

                        if (it.success) {
                            page = 1
                            isLast = false
                            getCommentList(contentId)
                        } else {
                            if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue(
                                    "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                )
                            }
                        }
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }
}
